/* 
   Unix SMB/Netbios implementation.
   Version 1.9.
   Main SMB server routines
   Copyright (C) Andrew Tridgell 1992-1998
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "includes.h"
#include "rpc_parse.h"

fstring pipe_name;

pstring servicesf = CONFIGFILE;
extern pstring debugf;
extern BOOL append_log;

/*****************************************************************************
 initialise srv_auth_fns array
 *****************************************************************************/
static void auth_init(rpcsrv_struct *l)
{
}

/*************************************************************************
 initialise an msrpc service
 *************************************************************************/
static void service_init(char* service_name)
{
	add_msrpc_command_processor( pipe_name, service_name, api_brs_rpc );
	generate_wellknown_sids();
}

/****************************************************************************
  reload the services file
  **************************************************************************/
static BOOL reload_msrpc(BOOL test)
{
	BOOL ret;

	if (lp_loaded()) {
		pstring fname;
		pstrcpy(fname,lp_configfile());
		if (file_exist(fname,NULL) && !strcsequal(fname,servicesf)) {
			pstrcpy(servicesf,fname);
			test = False;
		}
	}

	reopen_logs();

	if (test && !lp_file_list_changed())
		return(True);

	lp_killunused(NULL);

	ret = lp_load(servicesf,False,False,True);

	/* perhaps the config filename is now set */
	if (!test)
		reload_msrpc(True);

	reopen_logs();

	load_interfaces();

	return(ret);
}

/****************************************************************************
  main program
****************************************************************************/
static int main_init(int argc,char *argv[])
{
#ifdef HAVE_SET_AUTH_PARAMETERS
	set_auth_parameters(argc,argv);
#endif

#ifdef HAVE_SETLUID
	/* needed for SecureWare on SCO */
	setluid(0);
#endif

	append_log = True;

	TimeInit();

	setup_logging(argv[0],False);
	fstrcpy(pipe_name, "browser");
	slprintf(debugf, sizeof(debugf), "%s/log.%s", LOGFILEBASE, pipe_name);

	return 0;
}

static msrpc_service_fns fn_table =
{
	auth_init,
	service_init,
	reload_msrpc,
	main_init,
	NULL
};

msrpc_service_fns *get_service_fns(void)
{
	return &fn_table;
}

/*******************************************************************
 create_brs_info_100
 ********************************************************************/
static void create_brs_info_100(BRS_INFO_100 *inf)
{
	make_brs_info_100(inf);
}

/*******************************************************************
 _brs_query_info
 
 only supports info level 100 at the moment.

 ********************************************************************/
uint32 _brs_query_info( const UNISTR2 *srv_name, uint16 switch_value,
			void *id)
{
	create_brs_info_100(id);

	return 0x0;
}
